/*
 * The contents of this file are subject to the Mozilla Public License
 * Version 1.1 (the "License");  you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for
 * the specific language governing rights and limitations under the License.
 *
 * Copyright (C) 2004 Sorokin R.P. (sorokinru@iias.spb.su)
 * and Object-oriented Geoinformatics Systems Laboratory (http://niggis.iias.spb.su)
 * All Rights Reserved.
 */

package ru.igis.omtab;

import java.util.*;
import java.awt.Component;
import java.awt.Container;
import java.lang.reflect.*;
import com.bbn.openmap.geo.*;
import com.bbn.openmap.proj.*;
import com.bbn.openmap.proj.coords.LatLonPoint;

import edu.stanford.smi.protege.model.*;
import edu.stanford.smi.protege.ui.ProjectManager;
import edu.stanford.smi.protege.ui.ProjectView;
import edu.stanford.smi.protege.widget.TabWidget;
/**
 * OpenMapTab utilities class
 * @author Ru
 */
public class Util {
    
    /** Creates a new instance of Util */
    public Util() {
    }
    
    /**
     * Filter instances of only one class
     * @param col - Collection of instances
     * @param cls - Needed class
     * @return - Collection of instances
     */    
    @SuppressWarnings({ "unchecked", "rawtypes" })
	public static Collection filter(Collection col,Cls cls){
        Collection res = new ArrayList();
        for(Iterator i=col.iterator();i.hasNext();){
            Instance inst = (Instance)i.next();
            Collection types = inst.getDirectTypes();
            if(types.contains(cls))
                res.add(inst);
        }
        return res;
    }
    
    /**
     * Intersection point with polygon along the course
     * @param lat1 - Latitude of current point (degrees)
     * @param lon1 - Longitude of current point (degrees)
     * @param course - Current course (degrees)
     * @param llp - Array of lat lon points of polygon (radians)
     * @param givePoint - Flag if gives up point or only check fact of intersection
     * @return [-1,-1,Float.MAX_VALUE,-1] -if there is not an intersection point,
     * [0,-1,Float.MAX_VALUE,-1] -if there is an intersection point and givePoint=false,
     * [lat(degrees),lon(degrees),distance(NM),-1] -if there is an intersection point and givePoint=true.
     */    
    public static double[] intersectionPoint(double lat1,double lon1,int course,double[] llp,boolean givePoint){
        lat1 = ProjMath.degToRad(lat1);
        lon1 = ProjMath.degToRad(lon1);
        double crs = ProjMath.degToRad((double)course);
        double[] isp = new double[]{-1f,-1f,Float.MAX_VALUE,-1f};
        for(int i=0;i<llp.length;i+=2){
            double lat2 = llp[i];
            double lon2 = llp[i+1];
            double lat3,lon3;
            if(i<llp.length-2){
                lat3 = llp[i+2];
                lon3 = llp[i+3];
            } else if(llp.length<5){
                break;
            } else {
                lat3 = llp[0];
                lon3 = llp[1];
            }
            double bear1 = GreatCircle.sphericalAzimuth(lat1, lon1, lat2, lon2);
            // Azimuth calculated from zero in both directions (+/-)
            if (bear1<0)
                bear1 += Math.PI*2f;
            double bear2 = GreatCircle.sphericalAzimuth(lat1, lon1, lat3, lon3);
            if (bear2<0)
                bear2 += Math.PI*2f;
            double bmax = Math.max(bear1,bear2);
            double bmin = Math.min(bear1,bear2);
            if(
                (((bmax-bmin)<=Math.PI) && (bmin<=crs) && (crs<=bmax)) ||
                (((bmax-bmin)>Math.PI) && ((crs<bmin) || (bmax<crs)))
            ){
                // Our course do between two bearings
                if(!givePoint){
                    // Fast exit when only fact is needed
                    isp[0] = 0f;
                    return isp;
                }
                double[] point = getIntersectionPointRad(lat1,lon1,crs,lat2,lon2,lat3,lon3);
                if(point!=null){
                    double gcsd = GreatCircle.sphericalDistance(lat1, lon1, point[0], point[1]);
                    if(gcsd<isp[2]){
                        isp[0] = point[0];
                        isp[1] = point[1];
                        isp[2] = gcsd;
                    }
                }
            }
        }
        if(isp[0]>=0){
            isp[0] = ProjMath.radToDeg(isp[0]);
            isp[1] = ProjMath.radToDeg(isp[1]);
            isp[2] = isp[2] * Planet.wgs84_earthEquatorialCircumferenceNMiles
                            / 2f 
                            / (double)Math.PI;
        }
        return isp;
    }
    // All parameters are in radians
    private static double[] getIntersectionPointRad(double lat1,double lon1,double crs,double lat2,double lon2,double lat3,double lon3){
        double gcsd1 = GreatCircle.sphericalDistance(lat1, lon1, lat2, lon2);
        double gcsd2 = GreatCircle.sphericalDistance(lat1, lon1, lat3, lon3);
        double maxdi = Math.max(gcsd1,gcsd2);
        // Point along our course on a max from two distances
        LatLonPoint llp = GreatCircle.sphericalBetween(lat1, lon1, maxdi, crs);
        double lat4 = llp.getLatitude();
        double lon4 = llp.getLongitude();
        double[] point = Intersection.getSegIntersection(lat2,lon2,lat3,lon3,lat1,lon1,ProjMath.degToRad(lat4),ProjMath.degToRad(lon4));
        if(point[0]<Math.PI/2 && point[1]<Math.PI)
            return point;
        point[0] = point[2];
        point[1] = point[3];
        if(point[0]<Math.PI/2 && point[1]<Math.PI)
            return point;
        else
            return null;
    }

    /**
     * Method of execution of scenario
     * @param scenario - String in form "FULLCLASS METHOD PARAM1 PARAM2 ..."
     * @throws Exception -
     */
    @SuppressWarnings({ "unchecked", "rawtypes" })
	public static void execScenario(String scenario) throws Exception {
        String SYNTAX_ERROR = "SYNTAX ERROR in Scenario Call: ";
        StringQuoteTokenizer sqt = new StringQuoteTokenizer(scenario);
        if(sqt.hasMoreTokens()){
            String className = sqt.nextToken();
            if(sqt.hasMoreTokens()){
                String methodName = sqt.nextToken();
                ArrayList al = new ArrayList();
                // Parse parameters
                while(sqt.hasMoreTokens()){
                    String nt = sqt.nextToken().trim();
                    al.add(nt);
                }
                Object[] params = al.toArray();
                Class[] types = new Class[params.length];
                for(int i=0;i<types.length;i++)
                    try{
                        types[i] = Class.forName("java.lang.String");
                    } catch (Exception exc) {}
                Class c = Class.forName(className);
                if(c!=null){
                    Constructor constr = c.getConstructor(new Class[]{}); // constructor with 0 parameters
                    if(constr!=null){
                        Method method = c.getMethod(methodName,types);
                        if(method!=null){
                            Object[] const_params = new Object[]{};
                            Object obj = constr.newInstance(const_params);
                            method.invoke(obj,params);
                        } else
                            throw new Exception("ERROR, Method "+methodName+" not found!");
                    } else
                        throw new Exception("ERROR, Constructor "+className+" not found!");
                } else
                    throw new Exception("ERROR, Class "+className+" not found!");
            } else
                throw new Exception(SYNTAX_ERROR+scenario);
        } else
            throw new Exception(SYNTAX_ERROR+scenario);
    }

    static class StringQuoteTokenizer {
        private String delim = " \n\t\"";
        private StringTokenizer st;
 
        /** Creates a new instance of StringQuoteTokenizer */
        public StringQuoteTokenizer(String str) {
            st = new StringTokenizer(str,delim,true);
        }

        public boolean hasMoreTokens(){
            return st.hasMoreTokens();
        }

        public String nextToken(){
            if(st.hasMoreTokens()){
                String next = st.nextToken(delim);
                if(next.length()>1 || delim.indexOf(next)<0)
                    return next;
                else if(next.equals("\"")){
                    String inQuoteToken = st.nextToken("\"\\");
                    String closeToken = st.nextToken("\"\\");
                    while(closeToken.equals("\\")){
                        inQuoteToken += st.nextToken("\"\\"); // this is the '"'
                        inQuoteToken += st.nextToken("\"\\");
                        closeToken = st.nextToken("\"\\");
                    }
                    return inQuoteToken;
                }else
                    return nextToken();
            } else
                return null;
        }
    }
    
    /**
     * Point on some azimuth and distance from some point
     * @param lat - latitude of initial point
     * @param lon - longitude of initial point
     * @param azdeg - azimuth of relative point
     * @param dist - distance of relative point from initial point
     * @return - array of two numbers - latitude and longitude
     */
    public static double[] relPos(double lat,double lon,double azdeg,double dist){
        double c = dist*NavOb.Pi10800; // dist in rad
        double az = ProjMath.degToRad(azdeg);
        LatLonPoint llp = GreatCircle.sphericalBetween(
            ProjMath.degToRad(lat), ProjMath.degToRad(lon), (double)c, (double)az
        );
        return new double[]{ llp.getLatitude(), llp.getLongitude() };
    }
    
    /**
     * Search for MapOb with a name like a label of Protege Instance
     * or create one from it and add to RuNA
     * @param inst Protege Instance
     * @param runa RuNA object
     * @return Map Object
     */
    public static MapOb findOrAdd(Instance inst, RuNA runa){
        MapOb mapOb = runa.getMapOb(inst);
        if(mapOb==null){
            mapOb = runa.addMapOb(inst);
        }
        return mapOb;
    }
    
    public static TabWidget getTab(String fullClassName){
    	ProjectManager pm = ProjectManager.getProjectManager();
    	ProjectView pv = pm.getCurrentProjectView();
    	return pv.getTabByClassName(fullClassName);
    }
    
    public static Component getComponentOfClass(Container container, String className) {
        Component[] comps = container.getComponents();
        for (int i=0;i<comps.length;i++){
            if(comps[i].getClass().getName().equals(className))
                return comps[i];
            try {
                Component inner = getComponentOfClass((Container)comps[i], className);
                if(inner != null)
                	return inner;
			} catch (Exception e) {
			}
        }
        return null;
    }
    
    /*
    public static void main(String[] args) throws Exception {
        double[] rp = relPos(60f,30f,360f,60f);
        System.out.println("fp= "+rp[0]+" "+rp[1]);
    }
    */ 
    public static void printRadArrayInDeg(double[] rad_array, int how_many){
    	how_many = (how_many==0) ? rad_array.length : how_many * 2;
    	for(int i=0; i<how_many; i+=2){
    		System.out.println(i/2+" ["+ProjMath.radToDeg(rad_array[i])+", "+ProjMath.radToDeg(rad_array[i+1])+"]");
    	}
    }
}
