/*
 * The contents of this file are subject to the Mozilla Public License
 * Version 1.1 (the "License");  you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for
 * the specific language governing rights and limitations under the License.
 *
 * Copyright (C) 2004 Sorokin R.P. (sorokinru@iias.spb.su)
 * and Object-oriented Geoinformatics Systems Laboratory (http://niggis.iias.spb.su)
 * All Rights Reserved.
 */

package ru.igis.omtab.ra;

import ru.igis.omtab.*;
import edu.stanford.smi.protege.model.*;
/**
 * Abstract reference realisation of Repeat Action interface
 * Must be extended by concrete classes corresponding
 * to subclasses of class RepeatAction in Protege
 * @author ru
 */
public abstract class ProtegeRepeatAction implements RepeatAction {
    /**
     * Name of this Object
     */
    protected String name;
    /**
     * Time interval (SECOND,MINUTE or HOUR)
     */
    protected String interval;
    
    /** Creates a new instance of ProtegeRepeatAction */
    public ProtegeRepeatAction() {
    }
    
    /**
     * Method that implement repeating action
     * @param time elapsed time between calls (hours, minutes or seconds)
     */
    public abstract void repeat(long time);

    /**
     * Take information from Protege Instance
     * and put it to RepeatAction object
     * @param instance Instance of subclass of Protege RepeatAction class
     * @param kb Knowledge Base
     * @throws java.lang.Exception -
     * @param runa RuNA object
     */
    public abstract void fillFromProtege(Instance instance, KnowledgeBase kb, RuNA runa) 
        throws Exception;

    /**
     * Method to clear created while starting this RepeatAction object Map Objects
     * when removing it
     * @param runa RuNA object
     */
    public abstract void removeObjects(RuNA runa);
    
    /**
     * Name of this RepeatAction
     * @return String name
     */
    public String getName(){
        return name;
    }
    
    /**
     * Can be used to change name of RepeatAction
     * @param name new name
     */
    public void setName(String name){
        this.name = name;
    }
    
    /**
     * Add this RepeatAction object to corresponding list for execution priodically
     * depnding on interval (HOUR, MINUTE or SECOND)
     * @param runa RuNA object
     */
    public void addIntervalAction(RuNA runa){
        if(interval.equals(Ontology.SECOND))
            runa.addSecondAction(this);
        else if(interval.equals(Ontology.MINUTE))
            runa.addMinuteAction(this);
        else if(interval.equals(Ontology.HOUR))
            runa.addHourAction(this);
    }

    /**
     * Stop execution of Repeat Action by removing it 
     * from corresponding list
     * @param runa RuNA object
     */
    public void removeIntervalAction(RuNA runa){
        if(interval.equals(Ontology.SECOND))
            runa.removeSecondAction(this);
        else if(interval.equals(Ontology.MINUTE))
            runa.removeMinuteAction(this);
        else if(interval.equals(Ontology.HOUR))
            runa.removeHourAction(this);
    }
}
